from typing import Any, Optional
from langchain_core.embeddings import Embeddings
from langchain_core.language_models import BaseChatModel
from pydantic import BaseModel, ConfigDict, Field

class LlmConfig(BaseModel):
    api_key: str
    embedding_api_key: Optional[str] = None
    model: Optional[str] = None
    temperature: Optional[float] = Field(0.7, ge=0.0, le=2.0)

# Define an abstract LLM interface
class LlmInterface:
    def __init__(self, config: LlmConfig):
        self.config = config

    def get_llm(self) -> BaseChatModel:
        raise NotImplementedError

    def get_embeddings(self) -> Embeddings:
        from langchain_openai import OpenAIEmbeddings
        return OpenAIEmbeddings(model="text-embedding-3-small")

    def get_models(self) -> list:
        raise NotImplementedError

    def get_formatter(self,intermadiate_steps):
        from langchain.agents.format_scratchpad.openai_tools import format_to_openai_tool_messages
        return format_to_openai_tool_messages(intermediate_steps=intermadiate_steps)

    def get_parser(self):
        from langchain.agents.output_parsers.openai_tools import OpenAIToolsAgentOutputParser
        return OpenAIToolsAgentOutputParser()

class OpenAI(LlmInterface):
    def get_llm(self):
        from langchain_openai import ChatOpenAI
        chat = ChatOpenAI(api_key=self.config.api_key, model=self.config.model)
        if not any(self.config.model.startswith(prefix) for prefix in ["o1", "o3"]):
            chat.temperature = self.config.temperature
            chat.streaming = True
        return chat

    def get_embeddings(self):
        from langchain_openai import OpenAIEmbeddings
        return OpenAIEmbeddings(
            api_key=self.config.api_key,
            model="text-embedding-3-small")

    def get_models(self):
        import openai
        openai.api_key = self.config.api_key
        response = openai.models.list()
        return response.data

class DeepSeek(LlmInterface):
    def get_llm(self):
        from langchain_openai import ChatOpenAI
        return ChatOpenAI(
            api_key=self.config.api_key,
            model=self.config.model,
            base_url="https://api.deepseek.com/v1",
            max_tokens=8192,
            temperature=self.config.temperature,
            streaming=True,
        )

    def get_embeddings(self):
        from langchain_openai import OpenAIEmbeddings
        return OpenAIEmbeddings(
            api_key=self.config.embedding_api_key,
            model="text-embedding-3-small")

    def get_models(self):
        return [
              {"id":"deepseek-chat"},
              {"id":"deepseek-reasoner"}
            ]

class Google(LlmInterface):
  def get_llm(self):
    from langchain_google_genai.chat_models import ChatGoogleGenerativeAI
    return ChatGoogleGenerativeAI(
      name="chat",
      api_key=self.config.api_key,
      model=self.config.model,
      temperature=self.config.temperature,
      disable_streaming=False
    )

  def get_embeddings(self):
    from langchain_google_genai.embeddings import GoogleGenerativeAIEmbeddings
    return GoogleGenerativeAIEmbeddings(
      google_api_key=self.config.api_key,
      model="models/text-embedding-004")

  def get_models(self):
    import google.generativeai as genai
    genai.configure(api_key=self.config.api_key)
    response = genai.list_models()
    return [{
      "id": model.name,
      "name": model.display_name,
      "description": model.description,
      "input_token_limit": model.input_token_limit,
      "output_token_limit": model.output_token_limit
    } for model in response if "gemini" in model.name.lower()]

class Gvertex(LlmInterface):
    def get_llm(self):
        from langchain_google_vertexai  import ChatVertexAI
        return ChatVertexAI(
            model=self.config.model,
            temperature=self.config.temperature,
        )
    def get_embeddings(self):
        from langchain_google_vertexai import VertexAIEmbeddings
        return VertexAIEmbeddings(model_name="text-embedding-004")
    def get_models(self):
        from google.cloud import aiplatform
        aiplatform.init()
        models = aiplatform.Model.list()
        #see https://cloud.google.com/vertex-ai/generative-ai/docs/learn/locations#united-states for available models
        return models or [
              {"id":"gemini-2.0-flash-001"},
              {"id":"gemini-1.5-pro-001"},
              {"id":"gemini-1.5-pro-002"}
            ]

class Anthropic(LlmInterface):
    def get_llm(self):
        from langchain_anthropic import ChatAnthropic
        return ChatAnthropic(
            api_key=self.config.api_key,
            model=self.config.model,
            temperature=self.config.temperature,
            streaming=True,
            stream_usage=False
        )

    def get_embeddings(self):
        #from langchain_voyageai import VoyageAIEmbeddings
        #return VoyageAIEmbeddings(
        #    api_key=self.config.embedding_api_key, #voyage api key
        #    model="voyage-3")
        from langchain_openai import OpenAIEmbeddings
        return OpenAIEmbeddings(
            api_key=self.config.embedding_api_key,
            model="text-embedding-3-small")

    def get_models(self):
        import anthropic
        client = anthropic.Client(api_key=self.config.api_key)
        response = client.models.list()
        return response.data

    def get_formatter(self,intermadiate_steps):
        from langchain.agents.format_scratchpad.tools import format_to_tool_messages
        return format_to_tool_messages(intermediate_steps=intermadiate_steps)

    def get_parser(self):
        from langchain.agents.output_parsers.tools import ToolsAgentOutputParser
        return ToolsAgentOutputParser()

class LlmManager:

    #class variables (static)
    _list: dict[str,LlmInterface] = {
        "anthropic": Anthropic,
        "deepseek": DeepSeek,
        "google": Google,
        "gvertex": Gvertex,
        "openai": OpenAI,
    }
